<?php
/**
 * <pre>
 * Invision Power Services
 * IP.Board v3.1.0
 * Incoming Email Routing
 * Last Updated: $LastChangedDate: 2010-04-22 05:11:26 -0400 (Thu, 22 Apr 2010) $
 * </pre>
 *
 * @author 		$Author: mark $
 * @copyright	(c) 2001 - 2009 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/community/board/license.html
 * @package		IP.Board
 * @subpackage	Core
 * @link		http://www.invisionpower.com
 * @version		$Rev: 6161 $
 */

if ( ! defined( 'IN_ACP' ) )
{
	print "<h1>Incorrect access</h1>You cannot access this file directly. If you have recently upgraded, make sure you upgraded 'admin.php'.";
	exit();
}


class admin_core_tools_incomingEmails extends ipsCommand
{
	/**
	 * Main entry point
	 *
	 * @access	public
	 * @param	object		ipsRegistry reference
	 * @return	void
	 */
	public function doExecute( ipsRegistry $registry )
	{
		/* Load lang and skin */
		$this->registry->class_localization->loadLanguageFile( array( 'admin_tools' ) );
		$this->html = $this->registry->output->loadTemplate( 'cp_skin_tools' );
		
		$this->form_code    = $this->html->form_code = 'module=tools&amp;section=incomingEmails';
				
		/* What to do */
		switch( $this->request['do'] )
		{
			case 'add':
				$this->form('add');
				break;
				
			case 'edit':
				$this->form('edit');
				break;
				
			case 'save':
				$this->save();
				break;
				
			case 'delete':
				$this->delete();
				break;
		
			default:
				$this->manage();
				break;
		}
		
		/* Output */
		$this->registry->output->html_main .= $this->registry->output->global_template->global_frame_wrapper();
		$this->registry->output->sendOutput();
	}
	
	/**
	 * Manage Rules
	 *
	 * @access	public
	 * @return	void
	 */
	private function manage()
	{
		$rules = array();
		$this->DB->build( array( 'select' => '*', 'from' => 'core_incoming_emails' ) );
		$this->DB->execute();
		while ( $row = $this->DB->fetch() )
		{
			$rules[] = array(
				'id'		=> $row['rule_id'],
				'criteria'	=> $this->_getRuleText( $row['rule_criteria_field'], $row['rule_criteria_type'], $row['rule_criteria_value'] ),
				'action'	=> ( $row['rule_app'] == '--' ) ? $this->lang->words['ie_ignore'] : ( $this->lang->words['ie_sendto'] . ipsRegistry::$applications[ $row['rule_app'] ]['app_title'] )
				);
		}
	
		$this->registry->output->html .= $this->html->manageEmailRules( $rules );
	}
	
	/**
	 * Show Form
	 *
	 * @access	public
	 * @return	void
	 */
	private function form( $type )
	{
		$current = array();
		if ( $type == 'edit' )
		{
			$id = intval( $this->request['id'] );
			$current = $this->DB->buildAndFetch( array( 'select' => '*', 'from' => 'core_incoming_emails', 'where' => "rule_id={$id}" ) );
			if ( !$current['rule_id'] )
			{
				ipsRegistry::getClass('output')->showError( 'ie_not_found', 12345 );
			}			
		}
		
		$applications = array();
		foreach ( ipsRegistry::$applications as $dir => $app )
		{
			if ( file_exists( IPSLib::getAppDir( $dir ) . '/extensions/incomingEmails.php' ) )
			{
				$applications[] = array( $dir, $this->lang->words['ie_sendto'] . $app['app_title'] );
			}
		}
			
		$this->registry->output->html .= $this->html->emailRuleForm( $current, $applications );
	}
	
	/**
	 * Action: Save
	 */
	private function save()
	{			
		//-----------------------------------------
		// Save
		//-----------------------------------------
				
		$save = array( 
			'rule_criteria_field'	=> $this->request['criteria_field'],
			'rule_criteria_type'	=> $this->request['criteria_type'],
			'rule_criteria_value'	=> $this->request['criteria_value'],
			'rule_app'				=> $this->request['action'],
			'rule_added_by'			=> $this->memberData['member_id'],
			'rule_added_date'		=> time(),
			);
			
		if ( $this->request['id'] )
		{
			$id = intval( $this->request['id'] );
			$this->DB->update( 'core_incoming_emails', $save, "rule_id={$id}" );
			
			$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['ie_updated'], $this->_getRuleText( $save['rule_criteria_field'], $save['rule_criteria_type'], $save['rule_criteria_value'] ) ) );
		}
		else
		{
			$this->DB->insert( 'core_incoming_emails', $save );
			$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['ie_added'], $this->_getRuleText( $save['rule_criteria_field'], $save['rule_criteria_type'], $save['rule_criteria_value'] ) ) );
		}
		
		//-----------------------------------------
		// Display
		//-----------------------------------------
		
		$this->registry->output->global_message = $this->lang->words['ie_saved'];
		$this->manage();
	}
	
	/**
	 * Action: Delete
	 */
	private function delete()
	{
		$id = intval( $this->request['id'] );
		$current = $this->DB->buildAndFetch( array( 'select' => '*', 'from' => 'core_incoming_emails', 'where' => "rule_id={$id}" ) );
		if ( !$current['rule_id'] )
		{
			ipsRegistry::getClass('output')->showError( 'ie_not_found', 12345 );
		}
		
		$this->DB->delete( 'core_incoming_emails', "rule_id={$id}" );
		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['ie_deleted'], $this->_getRuleText( $current['rule_criteria_field'], $current['rule_criteria_type'], $current['rule_criteria_value'] ) ) );
				
		$this->registry->output->global_message = $this->lang->words['ie_delete_done'];
		$this->manage();
	}
	
	/**
	 * Return human-readable interpretation of rule
	 *
	 * @param	string	field
	 * @param	string	type
	 * @param	string	value
	 * @return	string
	 */
	private function _getRuleText( $field, $type, $value )
	{
		return $this->lang->words[ 'ie_cf_' . $field ] . ' ' . $this->lang->words[ 'ie_ct_' . $type ] . ' "' . $value . '"';
	}
}